import abc
from http import HTTPStatus

from src.core.DI import container
from src.core.logger import Logger
from src.core.logger.logger import LoggerHelper

from src.core.node import NodeServiceWrapper, NodeServiceParam


class ProcessorResponse(object):
    status: int = HTTPStatus.OK
    data: dict = {}

    def get(self, param: str = None):
        if param:
            return self.data.get(param, None)

        return self.data

    def set(self, data: dict):
        self.data = {**self.data, **data}

        return self


class Processor(abc.ABC):
    param: NodeServiceParam = container.param()
    logger: Logger = container.logger()
    prefix = {'node_name': 'processor', 'node_desc': 'Обработка состояния'}
    node_param_object: dict = {}

    @property
    @abc.abstractmethod
    def SCENARIOS(self) -> dict:
        """ Массив состояний """

    def run(self, scenario: str) -> ProcessorResponse:
        if scenario not in self.SCENARIOS.keys():
            self.logger.critical(f"Сценарий '{scenario}' не найден", prefix=self.prefix, result=LoggerHelper.FAILED)
            raise Exception(f"Сценарий '{scenario}' не найден")

        self.logger.info(f"Сценарий определён: '{scenario}'", prefix=self.prefix, result=LoggerHelper.PASSED)

        response = ProcessorResponse()
        self.param.set(ProcessorResponse.__name__, response)
        for key, value in self.node_param_object.items():
            self.param.set(key, value)

        self.logger.info(f"Запуск сценария '{scenario}'", prefix=self.prefix, result=LoggerHelper.PASSED)
        self.__handle_nodes(scenario)
        self.logger.info(f"Результат выполнения сценария '{scenario}'", prefix=self.prefix, result=LoggerHelper.PASSED)

        return response

    def add_node_param(self, key, value):
        self.node_param_object[key] = value

    def __handle_nodes(self, scenario):
        nodes = self.SCENARIOS.get(scenario)
        node = nodes['start']
        directions = nodes['nodes'].get(node, False)

        while node:
            result = NodeServiceWrapper(node()).handle()
            node = directions.get(result, False)
            if not node:
                break
            directions = nodes['nodes'].get(node, {})
